//
// Navigation button (written by "Dmitrij K" <kdiman at live dot ru>) for the Fast Light Tool Kit (FLTK).
//
// Copyright 1998-2010 by Bill Spitzak and others.
//
// This library is free software. Distribution and use rights are outlined in
// the file "COPYING" which should have been included with this file.  If this
// file is missing or damaged, see the license at:
//
//     http://www.fltk.org/COPYING.php
//
// Please report all bugs and problems on the following page:
//
//     http://www.fltk.org/str.php
//

#ifndef Fl_Nav_Button_H
#define Fl_Nav_Button_H

#include "Fl_Button.H"
#include "Fl_Repeat_Button.H"
#include "fl_draw.H"

extern FL_EXPORT void oxy_arrow(int x1, int y1, Fl_Align a, bool ac, Fl_Color c, Fl_Color hc=FL_BLACK);

class Fl_Nav_Repeat_Button : public Fl_Repeat_Button {
 Fl_Color _hlcol;
 bool _light;
 Fl_Align _al;
 bool _dbl; // double arrow ?
public:
 // get/set double arrow
 inline bool get_da(){ return _dbl; }
 inline void set_da(bool f){ _dbl = f; Fl_Repeat_Button::align(_al); }
 inline void light(bool flag){ if(!(_light = flag)){ _hlcol = FL_BLACK; } }
 inline bool light(){ return _light; }
 inline Fl_Nav_Repeat_Button(int x, int y, int w, int h, bool dbl=false):Fl_Repeat_Button(x,y,w,h,0) {
  _dbl = dbl;
  _hlcol = FL_BLACK;
  _light = false;
  align(FL_ALIGN_BOTTOM);
  box(FL_THIN_UP_BOX);
  down_box(FL_THIN_DOWN_BOX);
 }
 inline void align(Fl_Align a){
  bool flag_oxy = (Fl::scheme() && !strcmp(Fl::scheme(), "oxy"));
  if(a == FL_ALIGN_TOP || a == FL_ALIGN_BOTTOM || a == FL_ALIGN_RIGHT || a == FL_ALIGN_LEFT){ _al = a; }
  if(flag_oxy){ Fl_Repeat_Button::align(a); label(""); return; }
  switch(a){
    case FL_ALIGN_TOP:{
     Fl_Repeat_Button::align(FL_ALIGN_CENTER);
     if(_dbl) Fl_Repeat_Button::label("@-42<<");
     else Fl_Repeat_Button::label("@-42<");
     break;
    }
    case FL_ALIGN_BOTTOM:{
     Fl_Repeat_Button::align(FL_ALIGN_CENTER);
     if(_dbl) Fl_Repeat_Button::label("@-42>>");
     else Fl_Repeat_Button::label("@-42>");
     break;
    }
    case FL_ALIGN_RIGHT:{
     Fl_Repeat_Button::align(FL_ALIGN_CENTER);
     if(_dbl) Fl_Repeat_Button::label("@-45>>");
     else Fl_Repeat_Button::label("@-45>");
     break;
    }
    case FL_ALIGN_LEFT:{
     Fl_Repeat_Button::align(FL_ALIGN_CENTER);
     if(_dbl) Fl_Repeat_Button::label("@-45<<");
     else Fl_Repeat_Button::label("@-45<");
     break;
    }
    default:{ Fl_Repeat_Button::align(a); break; }
   }
 }
 inline virtual void draw(){
  bool flag_oxy = (Fl::scheme() && !strcmp(Fl::scheme(), "oxy"));
  Fl_Align a = Fl_Repeat_Button::align();
  
  if(!flag_oxy){
   align(a);
   Fl_Repeat_Button::draw();
   return;
  }
  //else if(*label()){ label(""); }
  
  //Fl_Color hlcol = 0x0;
  int dx = Fl::box_dx(FL_DOWN_BOX);
  //int dy = Fl::box_dy(FL_DOWN_BOX);
  int W = w();
  int H = h();
  int x1;
  int y1;
  
  if (type() == FL_HIDDEN_BUTTON) return;
  Fl_Color col = value() ? selection_color() : color();
  draw_box(value() ? (down_box()?down_box():fl_down(box())) : box(), col);
  draw_backdrop();
  
  switch(_al){
   case FL_ALIGN_TOP:{
    x1 = x()+W/2-4;
    y1 = y()+H/2+1;
    if(_dbl){
     oxy_arrow(x1, y1-2, _al, active_r(), color(), _hlcol);
     oxy_arrow(x1, y1+2, _al, active_r(), color(), _hlcol);
    } else { oxy_arrow(x1, y1, _al, active_r(), color(), _hlcol); }
    break;
   }
   
   case FL_ALIGN_BOTTOM:{
    L_BOTTOM:
    x1 = x()+W/2-4;
    y1 = y()+H/2-3;
    if(_dbl){
     oxy_arrow(x1, y1-2, _al, active_r(), color(), _hlcol);
     oxy_arrow(x1, y1+2, _al, active_r(), color(), _hlcol);
    } else { oxy_arrow(x1, y1, _al, active_r(), color(), _hlcol); }
    break;
   }
   
   case FL_ALIGN_LEFT:{
    x1 = x()+W/2+2;
    y1 = y()+H/2-4-1;
    if(_dbl){
     oxy_arrow(x1-2, y1, _al, active_r(), color(), _hlcol);
     oxy_arrow(x1+2, y1, _al, active_r(), color(), _hlcol);
    } else { oxy_arrow(x1, y1, _al, active_r(), color(), _hlcol); }
    break;
   }
   
   case FL_ALIGN_RIGHT:{
    x1 = x()+W/2-3+dx/2;
    y1 = y()+H/2-4-1;
    if(_dbl){
     oxy_arrow(x1-2, y1, _al, active_r(), color(), _hlcol);
     oxy_arrow(x1+2, y1, _al, active_r(), color(), _hlcol);
    } else { oxy_arrow(x1, y1, _al, active_r(), color(), _hlcol); }
    break;
   }
   default: { goto L_BOTTOM; break; }
  }
 }
 
 virtual int handle(int event)
    {
     if(!_light){ return Fl_Repeat_Button::handle(event); }
     bool flag_oxy = (Fl::scheme() && !strcmp(Fl::scheme(), "oxy"));
     if(!flag_oxy){ return Fl_Repeat_Button::handle(event); }
     
        switch (event)
        {
        case FL_ENTER:
            _hlcol = fl_color_average(color(), FL_BLACK, 0.55);
            Fl_Repeat_Button::redraw();
            break;
        case FL_LEAVE:
            _hlcol = FL_BLACK;
            Fl_Repeat_Button::redraw();
            break;
        }
        return Fl_Repeat_Button::handle(event);
    }
};






class Fl_Nav_Button : public Fl_Button {
 Fl_Color _hlcol;
 bool _light;
 Fl_Align _al;
 bool _dbl; // double arrow ?
public:
 // get/set double arrow
 inline bool get_da(){ return _dbl; }
 inline void set_da(bool f){ _dbl = f; Fl_Button::align(_al); }
 inline void light(bool flag){ if(!(_light = flag)){ _hlcol = FL_BLACK; } }
 inline bool light(){ return _light; }
 inline Fl_Nav_Button(int x, int y, int w, int h, bool dbl=false):Fl_Button(x,y,w,h,0) {
  _dbl = dbl;
  _hlcol = FL_BLACK;
  _light = false;
  align(FL_ALIGN_BOTTOM);
  box(FL_THIN_UP_BOX);
  down_box(FL_THIN_DOWN_BOX);
 }
 inline void align(Fl_Align a){
  bool flag_oxy = (Fl::scheme() && !strcmp(Fl::scheme(), "oxy"));
  if(a == FL_ALIGN_TOP || a == FL_ALIGN_BOTTOM || a == FL_ALIGN_RIGHT || a == FL_ALIGN_LEFT){ _al = a; }
  if(flag_oxy){ Fl_Button::align(a); label(""); return; }
  switch(a){
    case FL_ALIGN_TOP:{
     Fl_Button::align(FL_ALIGN_CENTER);
     if(_dbl) Fl_Button::label("@-42<<");
     else Fl_Button::label("@-42<");
     break;
    }
    case FL_ALIGN_BOTTOM:{
     Fl_Button::align(FL_ALIGN_CENTER);
     if(_dbl) Fl_Button::label("@-42>>");
     else Fl_Button::label("@-42>");
     break;
    }
    case FL_ALIGN_RIGHT:{
     Fl_Button::align(FL_ALIGN_CENTER);
     if(_dbl) Fl_Button::label("@-45>>");
     else Fl_Button::label("@-45>");
     break;
    }
    case FL_ALIGN_LEFT:{
     Fl_Button::align(FL_ALIGN_CENTER);
     if(_dbl) Fl_Button::label("@-45<<");
     else Fl_Button::label("@-45<");
     break;
    }
    default:{ Fl_Button::align(a); break; }
   }
 }
 inline virtual void draw(){
  bool flag_oxy = (Fl::scheme() && !strcmp(Fl::scheme(), "oxy"));
  Fl_Align a = Fl_Button::align();
  
  if(!flag_oxy){
   align(a);
   Fl_Button::draw();
   return;
  }
  //else if(*label()){ label(""); }
  
  //Fl_Color hlcol = 0x0;
  int dx = Fl::box_dx(FL_DOWN_BOX);
  //int dy = Fl::box_dy(FL_DOWN_BOX);
  int W = w();
  int H = h();
  int x1;
  int y1;
  
  if (type() == FL_HIDDEN_BUTTON) return;
  Fl_Color col = value() ? selection_color() : color();
  draw_box(value() ? (down_box()?down_box():fl_down(box())) : box(), col);
  draw_backdrop();
  
  switch(_al){
   case FL_ALIGN_TOP:{
    x1 = x()+W/2-4;
    y1 = y()+H/2+1;
    if(_dbl){
     oxy_arrow(x1, y1-2, _al, active_r(), color(), _hlcol);
     oxy_arrow(x1, y1+2, _al, active_r(), color(), _hlcol);
    } else { oxy_arrow(x1, y1, _al, active_r(), color(), _hlcol); }
    break;
   }
   
   case FL_ALIGN_BOTTOM:{
    L_BOTTOM:
    x1 = x()+W/2-4;
    y1 = y()+H/2-3;
    if(_dbl){
     oxy_arrow(x1, y1-2, _al, active_r(), color(), _hlcol);
     oxy_arrow(x1, y1+2, _al, active_r(), color(), _hlcol);
    } else { oxy_arrow(x1, y1, _al, active_r(), color(), _hlcol); }
    break;
   }
   
   case FL_ALIGN_LEFT:{
    x1 = x()+W/2+2;
    y1 = y()+H/2-4-1;
    if(_dbl){
     oxy_arrow(x1-2, y1, _al, active_r(), color(), _hlcol);
     oxy_arrow(x1+2, y1, _al, active_r(), color(), _hlcol);
    } else { oxy_arrow(x1, y1, _al, active_r(), color(), _hlcol); }
    break;
   }
   
   case FL_ALIGN_RIGHT:{
    x1 = x()+W/2-3+dx/2;
    y1 = y()+H/2-4-1;
    if(_dbl){
     oxy_arrow(x1-2, y1, _al, active_r(), color(), _hlcol);
     oxy_arrow(x1+2, y1, _al, active_r(), color(), _hlcol);
    } else { oxy_arrow(x1, y1, _al, active_r(), color(), _hlcol); }
    break;
   }
   default: { goto L_BOTTOM; break; }
  }
 }
 
 virtual int handle(int event)
    {
     if(!_light){ return Fl_Button::handle(event); }
     bool flag_oxy = (Fl::scheme() && !strcmp(Fl::scheme(), "oxy"));
     if(!flag_oxy){ return Fl_Button::handle(event); }
     
        switch (event)
        {
        case FL_ENTER:
            _hlcol = fl_color_average(color(), FL_BLACK, 0.55);
            Fl_Button::redraw();
            break;
        case FL_LEAVE:
            _hlcol = FL_BLACK;
            Fl_Button::redraw();
            break;
        }
        return Fl_Button::handle(event);
    }
};


#endif
