#ifndef Fl_Nav_Page_H
#define Fl_Nav_Page_H

#include <FL/Fl.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Nav_Button.H>
#include <FL/Fl_Input_Choice.H>
#include <FL/Fl_Choice.H>
//#include "Fl_Input_Choice2.H"
#include <FL/names.h>

class FL_EXPORT Fl_Nav_Page : public Fl_Group {
 Fl_Nav_Button *_nav[4];
 //Fl_Input_Choice2 _page;
 Fl_Choice _page;
 int _n_pages;
public:
 
 inline Fl_Nav_Button* button(int n){
  if(n >= 0 && n < 4){ return _nav[n]; }
  else { return 0; }
 }
 
 inline void set_size(int nsz){
  if(nsz == _n_pages){ return; }
  else if(nsz < 0){ nsz = 0; }
  _n_pages = nsz;
  init_size();
 }
 
 inline int get_size(){ return _n_pages; }
 
 inline int value(){
  return _page.value();
 }
 
 inline void value(int p){ _page.value(p); check(); }
 
 inline void do_callback(){ _page.do_callback(); }
 
 Fl_Nav_Page(int X, int Y, int W, int H, const char* L=0):
 Fl_Group(X,Y,W,H,L),
 _page(X+H*2, Y, W-H*4, H, 0){
  _nav[0] = new Fl_Nav_Button(X, Y, H, H, true); _nav[0]->align(FL_ALIGN_LEFT); _nav[0]->box(FL_UP_BOX);
  _nav[1] = new Fl_Nav_Button(X+H, Y, H, H, false); _nav[1]->align(FL_ALIGN_LEFT); _nav[1]->box(FL_UP_BOX);
  _nav[2] = new Fl_Nav_Button(X+W-H*2, Y, H, H, false); _nav[2]->align(FL_ALIGN_RIGHT); _nav[2]->box(FL_UP_BOX);
  _nav[3] = new Fl_Nav_Button(X+W-H, Y, H, H, true); _nav[3]->align(FL_ALIGN_RIGHT); _nav[3]->box(FL_UP_BOX);
  end();
  //Fl_Group::resizable(&_page);
  ///_page.input()->type(FL_INT_INPUT);
  //_page.when(FL_WHEN_CHANGED);
  _n_pages = 0;
  init_size();
  for(int _i = 0; _i < 4; _i++){ _nav[_i]->callback(&nav_cb, this); }
  //_page.callback(&page_cb, this);
  
  
 }
 
 void callback(void (*cb)(Fl_Widget*, void*), void *arg){ _page.callback(cb, arg); }
 
 inline void draw(){
  int X = x(), Y = y(), W = w(), H = h();
  fl_push_clip(X,Y,W,H);
  //draw_box();
  _nav[0]->resize(X, Y, H, H); draw_child(*_nav[0]);
  _nav[1]->resize(X+H, Y, H, H); draw_child(*_nav[1]);
  _page.resize(X+H*2, Y, W-H*4, H); draw_child(reinterpret_cast<Fl_Widget&>(_page)); ///_page.input()->redraw();
  _nav[2]->resize(X+W-H*2, Y, H, H); draw_child(*_nav[2]);
  _nav[3]->resize(X+W-H, Y, H, H); draw_child(*_nav[3]);
  fl_pop_clip();
 }
 
 
 inline void init_size(){
  _page.clear();
  if(_n_pages == 0){
   _page.add("0");
   _page.value(0);
  } else {
   for(int _i = 0; _i < _n_pages; _i++){
    char tmp[16];
    snprintf(tmp, 16, "%d", _i+1);
    _page.add(tmp);
   }
   _page.value(0);
   _page.activate();
  }
  check();
  
 }
 
 inline void check(){
  
  if(_n_pages <= 1){
   _nav[0]->deactivate();
   _nav[1]->deactivate();
   _page.deactivate();
   _nav[2]->deactivate();
   _nav[3]->deactivate();
   return;
  }
  
  int v = _page.value();
  if(v == 0){
   _nav[0]->deactivate();
   _nav[1]->deactivate();
  } else {
   _nav[0]->activate();
   _nav[1]->activate();
  }
  
  if(v == _n_pages-1){
   _nav[2]->deactivate();
   _nav[3]->deactivate();
  } else {
   _nav[2]->activate();
   _nav[3]->activate();
  }
  
 }
 
 static void nav_cb(Fl_Widget *w, void *d){
  Fl_Nav_Page *o=(Fl_Nav_Page *)d;
  if(o->get_size() == 0){ return; }
  if(w == o->button(0)){ o->value(0); o->do_callback(); }
  else if(w == o->button(3)){ o->value(o->get_size()-1); o->do_callback(); }
  else if(w == o->button(1)){ if(o->value() > 0){ o->value(o->value()-1); o->do_callback(); } }
  else if(w == o->button(2)){ if(o->value() < o->get_size()){ o->value(o->value()+1); o->do_callback(); } }
 }
 
 
 
 
};



#endif
